package kz.arta.ext.sap.service.proxy;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import kz.arta.ext.sap.entity.Department;
import kz.arta.ext.sap.entity.Group;
import kz.arta.ext.sap.entity.Position;
import kz.arta.ext.sap.entity.User;
import kz.arta.ext.sap.exception.GroupNotFoundException;
import kz.arta.ext.sap.exception.OwnerNotFoundException;
import kz.arta.ext.sap.service.ProxyEndpoints;
import kz.arta.ext.sap.util.HTTPRequestUtils;
import kz.arta.ext.sap.util.Json;
import lombok.AllArgsConstructor;
import lombok.NoArgsConstructor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;

@NoArgsConstructor
@AllArgsConstructor
public class ProxyService {
    private static final Logger LOGGER = LoggerFactory.getLogger(ProxyService.class);

    private String login;
    private String password;

    /**
     * Получение данных о пользователе
     * Get data about user
     *
     * @param userID
     * @return {@link User}
     * @throws Exception
     */
    public User getFilecabinetUserByID(String userID) throws Exception {

        LOGGER.info("sending a request to get user data");
        User user;
        String response = HTTPRequestUtils.sendGet(ProxyEndpoints.BASE_URL +
                ProxyEndpoints.FILECABINET_USER_ENDPOINT + userID, login, password);

        if (!response.equals("")) {
            user = new User();
            JsonElement parse = Json.serializer().parse(response);
            user.setUserid(parse.getAsJsonObject().get("userid").getAsString());
            user.setFirstname(parse.getAsJsonObject().get("firstname").getAsString());
            user.setLastname(parse.getAsJsonObject().get("lastname").getAsString());
            user.setPatronymic(parse.getAsJsonObject().get("patronymic").getAsString());
        } else {
            throw new OwnerNotFoundException(userID);
        }

        LOGGER.info("request complete");
        return user;

    }

    /**
     * Получение данных о департаменте
     * Get data about department
     *
     * @param departmentID
     * @return {@link Department}
     * @throws Exception
     */
    public Department getDepartmentByID(String departmentID) throws Exception {

        LOGGER.info("sending a request to get department data");
        Department department;
        String response = HTTPRequestUtils.sendGet(ProxyEndpoints.BASE_URL +
                ProxyEndpoints.GET_DEPARTMENT_ENDPOINT + "?departmentID=" + departmentID, login, password);

        if (!response.equals("")) {
            department = new Department();
            JsonElement parse = Json.serializer().parse(response);
            department.setDepartmentID(parse.getAsJsonObject().get("departmentID").getAsString());
            department.setNameRu(parse.getAsJsonObject().get("nameRu").getAsString());
        } else {
            throw new OwnerNotFoundException(departmentID);
        }

        if (department.getDepartmentID().equals("null")) throw new OwnerNotFoundException(departmentID);

        LOGGER.info("request complete");
        return department;

    }

    /**
     * Получение данных о должности
     * Get data about position
     *
     * @param positionID
     * @return
     * @throws Exception
     */
    public Position getPositionByID(String positionID) throws Exception {

        LOGGER.info("sending a request to get position data");
        Position position;
        String response = HTTPRequestUtils.sendGet(ProxyEndpoints.BASE_URL +
                ProxyEndpoints.GET_POSITION_ENDPOINT + "?positionID=" + positionID, login, password);

        if (!response.equals("")) {
            position = new Position();
            JsonElement parse = Json.serializer().parse(response);
            position.setPositionID(parse.getAsJsonObject().get("positionID").getAsString());
            position.setNameRu(parse.getAsJsonObject().get("nameRu").getAsString());
        } else {
            throw new OwnerNotFoundException(positionID);
        }

        if (position.getPositionID().equals("null")) throw new OwnerNotFoundException(positionID);

        LOGGER.info("request complete");
        return position;

    }

    public HashMap<Integer, Group> getGroups() throws Exception {

        LOGGER.info("sending a request to get position data");
        HashMap<Integer, Group> map = new HashMap<>();
        String response = HTTPRequestUtils.sendGet(ProxyEndpoints.BASE_URL +
                ProxyEndpoints.FIND_GROUP_ENDPOINT, login, password);

        if (!response.equals("")) {
            JsonElement parse = Json.serializer().parse(response);
            parse.getAsJsonObject().get("array").getAsJsonArray().forEach(item -> {
                JsonObject jsonObject = item.getAsJsonObject();
                Group group = new Group();
                group.setGroupID(jsonObject.getAsJsonObject().get("groupID").getAsInt());
                group.setName(jsonObject.getAsJsonObject().get("name").getAsString());
                map.put(group.getGroupID(), group);
            });
        }

        LOGGER.info("request complete");
        return map;

    }
}
