package kz.arta.ext.sap.util;

import com.google.gson.*;
import kz.arta.ext.sap.State;
import kz.arta.ext.sap.entity.*;
import kz.arta.ext.sap.exception.GroupNotFoundException;
import kz.arta.ext.sap.exception.OwnerNotFoundException;
import kz.arta.ext.sap.service.ProxyEndpoints;
import kz.arta.ext.sap.service.proxy.ProxyService;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.PreDestroy;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.inject.Inject;
import java.io.FileInputStream;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ImportBean {

    private static final ExecutorService executor;

    static {
        executor = Executors.newFixedThreadPool(15);
    }

    private static final Logger LOGGER = LoggerFactory.getLogger(ImportBean.class);
    private static final String NO = "no";

    private static Map<String, String> defaultValues = new HashMap<>();
    private static Map<String, String> searchValues = new HashMap<>();

    public static volatile State importState;

    @TransactionAttribute(TransactionAttributeType.REQUIRED)
    public void getFormFieldForRowAsync(final Row row, final FormulaEvaluator evaluator, final Map<String, FormField> formDefinitionMap, final String registryCode, final Boolean isActivate, final String login, final String password, final int action, final boolean shouldPasteMeaningValue) {
        executor.submit(new Runnable() {
            @Override
            public void run() {
                importState.count.incrementAndGet();
                getFormFieldForRow(row, evaluator, formDefinitionMap, registryCode, login, password, isActivate, action, shouldPasteMeaningValue);
            }
        });
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRED)
    public static State parseFile(String filename, String formcode, String registryCode, Boolean isActivate, String login, String password, int action, String defaultValue, String searchString, Boolean shouldPasteMeaningValue) throws Exception {
        try {
            LOGGER.info("Start parsing file: " + filename);

            //рид файл

            if (defaultValue != null && !defaultValue.equals("none")) {
                String[] defValues = defaultValue.split(", ");
                for (int i = 0; i < defValues.length; i++) {
                    String[] val = defValues[i].split(":");
                    defaultValues.put(val[0], val[1]);
                }
            }

            if (searchString != null && !searchString.equals("none")) {
                String[] ss = searchString.split(", ");
                for (int i = 0; i < ss.length; i++) {
                    String[] val = ss[i].split(":");
                    searchValues.put(val[0], val[1]);
                }
            }

            //получаем formDefinition
            String formDefinition = getFormDefinition(formcode, login, password);
            JsonParser parser = new JsonParser();
            JsonElement jsonTree = parser.parse(formDefinition);
            if (jsonTree.isJsonObject()) {

                JsonObject formJson = jsonTree.getAsJsonObject();
                Map<String, FormField> formDefinitionMap = getFormDefinitionMap(formJson, login, password);

                HSSFWorkbook importedFileWB = new HSSFWorkbook(new FileInputStream(filename));
                FormulaEvaluator evaluator = importedFileWB.getCreationHelper().createFormulaEvaluator();

                for (int i = 0; i < importedFileWB.getNumberOfSheets(); i++) {
                    Sheet importedFileSheet = importedFileWB.getSheetAt(i);
                    List<Row> sheetRows = getAllSheetRows(importedFileSheet);

                    importState = new State(sheetRows.size());


                    for (Row row : sheetRows) {
                        try {
                            ImportBean iu = new ImportBean();
                            iu.getFormFieldForRowAsync(row, evaluator, formDefinitionMap, registryCode, isActivate, login, password, action, shouldPasteMeaningValue);

                        } catch (Exception e) {
                            importState.errors.add(e.getMessage() + "\\n");
                            LOGGER.error(e.getMessage());
                        }
                    }

                }


            } else {
                throw new Exception("Wrong form definition! " + jsonTree.getAsString());
            }

            return importState;
        } catch(Exception e){
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    private static String formateRequestData(List<FormField> data, String registryCode){
        try {
            String jsonData = "[";
            int i = 0;
            for (FormField ff : data) {
                //добавляем поля в json

                jsonData += "{ \"id\": \"" + ff.getCode() + "\", \"type\": \"" + ff.getType() + "\"";

                //если это словарь и не чек бокс, нужно просто задать ему значение key
                if (ff.getType().equals(FormFieldType.LISTBOX.value)) {

                    jsonData += ", \"key\": \"" + ff.getKey() + "\",  \"value\": \"" + ff.getValue() + "\"";

                } else if (ff.getType().equals(FormFieldType.RADIO.value)) {

                    jsonData += ", \"key\": \"" + ff.getValue() + "\",  \"value\": \"" + ff.getKey() + "\"";

                } else if (ff.getType().equals(FormFieldType.CHECKBOX.value)) {

                    jsonData += ", \"values\": [";
                    for (String key : ff.getKeys()) {
                        jsonData += "\"" + key + "\"";

                        if (key != ff.getKeys().get(ff.getKeys().size() - 1)) {
                            jsonData += ", ";
                        }
                    }

                    jsonData += "], \"keys\": [";
                    for (String val : ff.getValues()) {
                        jsonData += "\"" + val + "\"";

                        if (val != ff.getValues().get(ff.getValues().size() - 1)) {
                            jsonData += ", ";
                        }
                    }

                    jsonData += "]";


                } else if (ff.getType().equals(FormFieldType.NUMERICINPUT.value)) {

                    jsonData += ", \"key\": \"" + ff.getValue() + "\",  \"value\": \"" + ff.getValue() + "\"";

                } else if (ff.getType().equals(FormFieldType.REGLINK.value)) {

                    if (ff.getKey() != null) {
                        jsonData += ", \"key\": \"" + ff.getKey() + "\",  \"value\": \"" + ff.getValue() + "\", \"valueID\": \"" + ff.getKey() + "\"";
                    }

                } else if (ff.getType().equals(FormFieldType.ENTITY.value)) {

                    jsonData += ", \"key\": \"" + ff.getKey() + "\", \"value\": \"" + ff.getValue() + "\"";

                } else if (ff.getType().equals(FormFieldType.DATE.value)) {

                    String value = formatDate(ff.getDateFormat(), ff.getKey());
                    if(ff.getDateFormat().contains("h"))
                        jsonData += ", \"key\": \"" + ff.getKey() + "\",  \"value\": \"" + value + "\"";
                    else {
                        String key = ff.getKey().split(" ")[0];
                        jsonData += ", \"key\": \"" + key + "\",  \"value\": \"" + value + "\"";

                    }
                } else { //если текстовое поле

                    jsonData += ", \"value\": \"" + ff.getValue() + "\"";

                }

                jsonData += "}";
                if (i != data.size() - 1) {
                    jsonData += ",";
                }
                i++;
            }

            if (("" + jsonData.charAt(jsonData.length() - 1)).equals(",")) {
                jsonData = jsonData.substring(0, jsonData.length() - 1);
            }
            jsonData += "]";

            String mainjson = "{ \"registryCode\": \"" + registryCode + "\", \"data\": " + jsonData + "}";

            return mainjson;
        } catch(Exception e){
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    private static List<Row> getAllSheetRows(Sheet importedFileSheet ){
        List<Row> rows = new ArrayList<>();
        try {
            Iterator<Row> rowIt = importedFileSheet.rowIterator();
            if (rowIt != null) {
                rowIt.next();  //пропускаем первый ряд с заголовками
            }
            while (rowIt.hasNext()) {
                Row row = rowIt.next();
                rows.add(row);
            }

            return rows;
        } catch (Exception e){
            return rows;
        }
    }

    private static String getFormDefinition(String formCode, String login, String password) throws Exception {
        if(formCode == null || formCode.equals("")) return null;
        String answer = HTTPRequestUtils.sendGet(ProxyEndpoints.BASE_URL + "rest/api/asforms/form_ext?formCode=" + formCode, login, password);
        return answer;
    }


    /**
     *
     * @param row
     * @param formFields  key - код поля формы, value - FormField c его данными из определения формы
     * @return
     */
    private static void getFormFieldForRow(Row row, FormulaEvaluator evaluator, Map<String, FormField> formFields, String registryCode, String login, String password, Boolean isActivate, int action, Boolean shouldPasteMeaningValue){
        try {

            List<FormField> rowFF = new ArrayList<>();
            Iterator<Cell> iter = row.cellIterator();
            String tempValue = null;
            while (iter.hasNext()) {
                Cell cell = iter.next();

                CellValue cellValue = evaluator.evaluate(cell);
                if(cellValue == null || cellValue.equals("")) continue;
                String header = cell.getSheet().getRow(0).getCell(cell.getColumnIndex()).getRichStringCellValue().toString();

                try {

                    //получили заголовок ячейки

                    if (header.equals(NO) || header.equals("")) {
                        continue;
                    }

                    FormField defField = formFields.get(header);
                    if (defField == null) {
                        importState.errors.add("Ошибка импорта, заголовок :" + header + " не найден в определении формы.\\n");
                        LOGGER.error("Import error in line :" + (row.getRowNum() + 1));
                    }
                    FormField valueFF = new FormField(defField);


                    if (cellValue == null) continue;
                    switch (cellValue.getCellType()) {
                        case NUMERIC:
                            String value = "";
                            try {
                                value = cellValue.getNumberValue() + "";
                            } catch (NullPointerException e){
                                value = cellValue.getStringValue();
                            }
                            if (defField.getType().equals(FormFieldType.DATE.value)) {
                                if (cell.getDateCellValue() == null) {
                                    continue;
                                }

                                // "key": "2019-08-29 11:07:00"
                                Date date = cell.getDateCellValue();
                                valueFF.setKey((date.getYear() + 1900) + "-" + ((date.getMonth() + 1) < 10 ? "0" + (date.getMonth() + 1) : (date.getMonth() + 1)) + "-" +
                                        (date.getDate() < 10 ? "0" + date.getDate() : date.getDate()) + " " +
                                        (date.getHours() < 10 ? "0" + date.getHours() : date.getHours()) + ":" +
                                        (date.getMinutes() < 10 ? "0" + date.getMinutes() : date.getMinutes()) + ":" +
                                        (date.getSeconds() < 10 ? "0" + date.getSeconds() : date.getSeconds()));
                            } else if (defField.getType().equals(FormFieldType.ENTITY.value)) {
                                valueFF.setValue(new DataFormatter().formatCellValue(cell));
                            } else {
                                valueFF.setValue(new BigDecimal(value).toPlainString());
                            }
                            try {
                                if (valueFF.isRegLink()) valueFF.setValue(value);
                            } catch (Exception ingnore){
                                valueFF.setValue(value);
                            }
                            break;
                        case STRING:
                            if (cellValue.getStringValue() == null || cellValue.getStringValue().equals("")) {
                                continue;
                            }

                            String val = cellValue.getStringValue();
                            if (val.equals("")) continue;
                            valueFF.setValue(unescapeJSONSpecials(val));
                            break;
                        case BLANK:
                            continue;
                        case BOOLEAN:
                            valueFF.setValue(cellValue.getBooleanValue() + "");
                            break;
                    }

                    if (valueFF.isDictionary()) {
                        //из карты со значениями словаря берем список элементов и ложим его значение в key

                        if (valueFF.getType().equals(FormFieldType.CHECKBOX.value)) {

                            String[] values = valueFF.getValue().split("; ");
                            ArrayList<String> arlvals = new ArrayList<>();
                            ArrayList<String> keys = new ArrayList<>();
                            for (int i = 0; i < values.length; i++) {
                                arlvals.add(values[i]);
                                if (valueFF.getElements().get(values[i]) == null) {
                                    importState.errors.add("Ошибка в строке :" + (row.getRowNum() + 1) + ", в значении " + values[i] + ", значение не найдено в словаре.\\n");
                                    importState.imported.incrementAndGet();
                                    LOGGER.error("Import error in line :" + (row.getRowNum() + 1));
                                    return;
                                } else {
                                    keys.add(valueFF.getElements().get(values[i]));
                                }
                            }

                            valueFF.setValues(arlvals);
                            valueFF.setKeys(keys);

                        } else {
                            try {
                                valueFF.setKey(unescapeJSONSpecials(valueFF.getElements().get(valueFF.getValue())));
                            }catch (Exception ignore){}
                        }

                    } else if (valueFF.isEntity()) {
                        valueFF.setKey(valueFF.getValue());
                        ProxyService proxyService = new ProxyService(login, password);

                        String[] values = valueFF.getValue().split(";");
                        List<String> results = new ArrayList<>();

                        for (int i = 0; i < values.length; i++) {
                            tempValue = values[i].trim();
                            if (tempValue.startsWith("g-")) {
                                String groupId = tempValue.substring(2);
                                HashMap<Integer, Group> groups = proxyService.getGroups();
                                if (groups.containsKey(Integer.parseInt(groupId))) {
                                    results.add(groups.get(Integer.parseInt(groupId)).getName());
                                } else {
                                    throw new GroupNotFoundException(groupId);
                                }
                            } else if (tempValue.startsWith("text")) {
                                //TODO: не реализованно
                            } else {
                                if (EntityType.USERS.value.equals(valueFF.getEntityType())) {
                                    User user = proxyService.getFilecabinetUserByID(tempValue);
                                    results.add(user.getPersonName());
                                } else if (EntityType.POSITIONS.value.equals(valueFF.getEntityType())) {
                                    Position position = proxyService.getPositionByID(tempValue);
                                    results.add(position.getNameRu());
                                } else if (EntityType.DEPARTMENTS.value.equals(valueFF.getEntityType())) {
                                    Department department = proxyService.getDepartmentByID(tempValue);
                                    results.add(department.getNameRu());
                                }
                            }
                        }

                        if (EntityType.DEPARTMENTS.value.equals(valueFF.getEntityType())) {
                            valueFF.setValue(String.join(";; ", results));
                        } else {
                            valueFF.setValue(String.join(", ", results));
                        }

                    } else if (valueFF.isDinTable()) {
                        //TODO: не реализованно
                    } else if (valueFF.isRegLink()) {

                        String documentID = "";
                        try {
                            documentID = getRegLinkObject(valueFF.getRegistryID(), valueFF.getCode(), valueFF.getValue(), action, login, password);
                        }catch (Exception e){
                            importState.errors.add("Ошибка в ссылке на реестр со значением : " + valueFF.getValue() + ", объект с данным значением не найден. \\n");
                        }
                        if(shouldPasteMeaningValue){
                            //если в value нужно вставить значимое содержимое реестра, то вставляем новый value
                            valueFF.setValue(unescapeJSONSpecials(getMeaningContentForRegLink(documentID, login, password)));
                        }
                        if (documentID != null) valueFF.setKey(documentID);

                    }

                    rowFF.add(valueFF);
                } catch (GroupNotFoundException e) {
                    importState.errors.add("Ошибка в строке :" + (row.getRowNum() + 1) +
                            ", группа с id = " + tempValue + " не найдена" + ". Заголовок: " + header + ". \\n");
                    importState.imported.incrementAndGet();
                    return;
                } catch (OwnerNotFoundException e) {
                    importState.errors.add("Ошибка в строке :" + (row.getRowNum() + 1) +
                            ", id = " + tempValue + " не найден" + ". Заголовок: " + header + ". \\n");
                    importState.imported.incrementAndGet();
                    return;
                } catch (Exception e){
                    importState.errors.add("Ошибка в строке :" + (row.getRowNum() + 1) +
                            ", со значением: " + cellValue.getStringValue() + ". Заголовок: " + header + ". \\n");
                    importState.imported.incrementAndGet();
                    LOGGER.error("Error in line :" + (row.getRowNum() + 1) + ". Something wrong with cell of type: " + cellValue.getCellType() +
                            ", and value: " + cellValue.formatAsString() + ". The header is: " + header + ".");
                    LOGGER.error(e.getMessage());
                    return;
                }
            }

            if (rowFF != null && !rowFF.isEmpty()) {
                String data = formateRequestData(rowFF, registryCode);
                if(validateJSON(data)) {
                    String answer = HTTPRequestUtils.sendPost(ProxyEndpoints.BASE_URL + "rest/api/registry/create_doc_rcc", data, login, password);
                    JsonParser answrPars = new JsonParser();
                    JsonElement jsonAnswr = answrPars.parse(answer);
                    String errorCode = jsonAnswr.getAsJsonObject().get("errorCode").getAsString();
                    if (!errorCode.equals("0")) {
                        importState.errors.add("Ошибка сохранения документа из строки " + (row.getRowNum() + 1) + "  " + jsonAnswr.getAsJsonObject().get("errorMessage").getAsString() + "\\n");
                        importState.imported.incrementAndGet();
                        LOGGER.error("CREATE DOC RCC error in line :" + (row.getRowNum() + 1) + ", data: " + data);
                        return;
                    } else {
                        if (isActivate) {
                            String dataUUID = jsonAnswr.getAsJsonObject().get("dataID").getAsString();
                            HTTPRequestUtils.sendGet(ProxyEndpoints.BASE_URL + "rest/api/registry/activate_doc?dataUUID=" + dataUUID, login, password);
                            importState.imported.incrementAndGet();
                        } else {
                            importState.imported.incrementAndGet();
                        }
                    }
                } else {
                    importState.errors.add("Сгенерирован некорректный JSON для сохранения!");
                    importState.imported.incrementAndGet();
                    importState.errors.add(data);
                }

            } else {
                importState.errors.add("Ошибка импорта в строке :" + (row.getRowNum() + 1) + " \\n");
                importState.imported.incrementAndGet();
                LOGGER.error("Import error in line :" + (row.getRowNum() + 1));
                return;
            }

        } catch (Exception e){
            importState.errors.add("Ошибка импорта в строке :" + (row.getRowNum() + 1) + " \\n");
            importState.imported.incrementAndGet();
            LOGGER.error("Import error in line :" + (row.getRowNum() + 1), e.getMessage());
            return;
        }

    }

    /**
     * Метод для получения значащего содержимого реестра, для получения корректного value для reglink
     * @return
     */
    public static String getMeaningContentForRegLink(String documentID, String login, String password) throws Exception{
        String meaningContent = HTTPRequestUtils.sendGet(ProxyEndpoints.BASE_URL + "rest/api/formPlayer/getDocMeaningContent?documentId=" + documentID, login, password);
        return meaningContent;
    }


    /**
     * Метод преобразует поля формы из form definition в Map<Код компонента формы, FormField>
     * @param formDefinition
     * @return
     */
    private static Map<String, FormField> getFormDefinitionMap(JsonObject formDefinition, String login, String password) throws Exception {
        try {
            Map<String, FormField> mapFormDefinition = new HashMap<>();
            //properties
            JsonArray properties = formDefinition.get("properties").getAsJsonArray();
            if (properties != null) {
                for (int i = 0; i < properties.size(); i++) {
                    JsonObject formField = properties.get(i).getAsJsonObject();
                    FormField ff = new FormField();
                    ff.setCode(formField.get("id").getAsString());
                    ff.setType(formField.get("type").getAsString());
                    ff.setRequired(formField.get("required").getAsBoolean());
                    if (ff.getType().equals(FormFieldType.CHECKBOX.value) || ff.getType().equals(FormFieldType.RADIO.value) ||
                            ff.getType().equals(FormFieldType.LISTBOX.value)) {

                        //Забиваем элементы справочника для этих полей в поле ff.elements
                        Map<String, String> elements = new HashMap<>();
                        if (formField.get("elements") != null) {
                            JsonArray elementsArray = formField.get("elements").getAsJsonArray();
                            if (elementsArray != null) {
                                for (int j = 0; j < elementsArray.size(); j++) {
                                    elements.put(elementsArray.get(j).getAsJsonObject().get("label").getAsString(), elementsArray.get(j).getAsJsonObject().get("value").getAsString());
                                }

                                ff.setElements(elements);
                            }
                        } else if (formField.get("dataSource") != null) {
                            try {
                                JsonObject dataSource = formField.get("dataSource").getAsJsonObject();
                                ff.setElements(getDictionary(dataSource.get("dict").getAsString(), dataSource.get("key").getAsString(), dataSource.get("value").getAsString(), login, password));
                            } catch (Exception e) {
                                LOGGER.error(e.getMessage());
                            }
                        }


                        ff.setDictionary(true);


                    } else if (ff.getType().equals(FormFieldType.DATE.value)) {

                        //Заполняем формат даты, если это поле типа дата
                        String dateFormat = formField.get("config").getAsJsonObject().get("dateFormat").getAsString();
                        ff.setDate(true);
                        ff.setDateFormat(dateFormat);

                    } else if (ff.getType().equals(FormFieldType.ENTITY.value)) {

                        //заполняем тип сущности, если это поле типа сущности
                        ff.setEntity(true);
                        ff.setEntityType(formField.get("config").getAsJsonObject().get("entity").getAsString());

                    } else if (ff.getType().equals(FormFieldType.REGLINK.value)) {

                        //заполняем код реестра если это ссылка на реестр
                        ff.setRegLink(true);
                        if(formField != null && formField.get("config") != null && formField.get("config").getAsJsonObject().get("dateFormat") != null)
                            ff.setRegistryID(formField.get("config").getAsJsonObject().get("dateFormat").getAsString());
                        else LOGGER.error("THERE IS NO REGISTRY SELECT FOR FORM COMPONENT CALLED " + ff.getCode());

                    } else if (ff.getType().equals(FormFieldType.TABLE.value)) {
                        mapFormDefinition.putAll(getFormDefinitionMap(formField, login, password));
                    }

                    if (!ff.getType().equals(FormFieldType.TABLE.value)) {
                        mapFormDefinition.put(ff.getCode(), ff);
                    }

                }
            }

            return mapFormDefinition;
        } catch (Exception e){
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    private static boolean validateJSON(String json){
        try {
            JsonParser parser = new JsonParser();
            JsonElement jsonTree = parser.parse(json);
            if(jsonTree == null) return false;
        } catch (Exception e){
            return false;
        }

        return true;
    }

    private static Map<String, String> getDictionary(String dictCode, String columnName, String columnValue, String login, String password) throws Exception {
        try {
            String dictionary = HTTPRequestUtils.sendGet(ProxyEndpoints.BASE_URL + "rest/api/dictionary/get_by_code?dictionaryCode=" + dictCode, login, password);
            JsonParser parser = new JsonParser();
            JsonElement jsonTree = parser.parse(dictionary);
            Map<String, String> elements = new HashMap<>();

            if (jsonTree.isJsonObject()) {

                JsonArray ja = jsonTree.getAsJsonObject().get("columns").getAsJsonArray();
                Map<String, String> columnIDsAndCodes = new HashMap<>();
                for (int i = 0; i < ja.size(); i++) {
                    if (ja.get(i).getAsJsonObject().get("code").getAsString().equals(columnName) || ja.get(i).getAsJsonObject().get("code").getAsString().equals(columnValue)) {
                        columnIDsAndCodes.put(ja.get(i).getAsJsonObject().get("code").getAsString(), ja.get(i).getAsJsonObject().get("columnID").getAsString());
                    }
                }

                JsonArray dictItems = jsonTree.getAsJsonObject().get("items").getAsJsonArray();
                for (int i = 0; i < dictItems.size(); i++) {
                    JsonArray valuesArray = dictItems.get(i).getAsJsonObject().get("values").getAsJsonArray();

                    String val = "", key = "";
                    for (int j = 0; j < valuesArray.size(); j++) {
                        String columnID = valuesArray.get(j).getAsJsonObject().get("columnID").getAsString();
                        if (columnID.equals(columnIDsAndCodes.get(columnName))) {
                            val = valuesArray.get(j).getAsJsonObject().get("value").getAsString();
                        } else if (columnID.equals(columnIDsAndCodes.get(columnValue))) {
                            key = valuesArray.get(j).getAsJsonObject().get("value").getAsString();
                        }
                    }
                    elements.put(val, key);
                }
            }

            return elements;
        } catch (Exception e){
            LOGGER.error(e.getMessage(), e);
            return null;
        }
    }

    private static String getRegLinkObject(String regID, String fieldCode, String value, int action, String login, String password) throws Exception {
        try {
            String searchField = searchValues.get(fieldCode);
            String answer = HTTPRequestUtils.sendGet(ProxyEndpoints.BASE_URL + "rest/api/registry/data_ext?registryID=" + regID + "&field=" + searchField
                    + "&condition=TEXT_EQUALS&value=" + URLEncoder.encode(value.replaceAll("\\\\\"", "\""), StandardCharsets.UTF_8.toString()), login, password);
            //парсим json, находим нужный объект и записываем его данные в систему
            JsonParser jp = new JsonParser();
            JsonElement jsonTree = jp.parse(answer);
            if (jsonTree.isJsonObject()) {
                JsonArray find = jsonTree.getAsJsonObject().get("result").getAsJsonArray();
                if (find.size() > 1) {
                    String docID = "";
                    switch (action) {
                        case 0: // EntitySearchActions.GET_DEFAULT
                            return defaultValues.get(fieldCode);
                        case 1: // EntitySearchActions.SET_EMPTY
                            return null;
                        case 2: // EntitySearchActions.TAKE_FIRST
                            docID = find.get(0).getAsJsonObject().get("documentID").getAsString();
                            break;
                    }

                    return docID;

                } else if (find.size() == 0) {
                    return null;
                } else {
                    if (find.get(0) != null) {
                        String docID = find.get(0).getAsJsonObject().get("documentID").getAsString();
                        return docID;
                    }
                }
            }
        } catch (Exception e){
            LOGGER.error(e.getMessage(), e);
        }
        return null;
    }

    public static String formatDate(String format, String date) {
        try {
            Date d = null;
            try {
                d = DateFormats.TIMESTAMP.parse(date);
            } catch (Exception e){
                LOGGER.error("WRONG DATE FORMAT IN XLS FILE! SHOULD BE yyyy-MM-dd hh:mm:ss");
                importState.errors.add("Неверный формат даты в файле! Должен быть гггг-мм-дд чч:мм:сс в формате ячейки типа Дата \\n");
                return "";
            }

            Matcher matcher = Pattern.compile("(\\$\\{.[^\\}\\{\\$]*\\})").matcher(format);

            Pattern partPattern = Pattern.compile("\\$\\{(.+)\\}");

            Calendar calendar = Calendar.getInstance();
            calendar.setTime(d);

            String result = format;
            while (matcher.find()) {
                String part = matcher.group(0);
                Matcher m = partPattern.matcher(part);
                if (!m.find()) {
                    continue;
                }
                String partName = m.group(1);
                result = result.replace(part, getDatePart(calendar, partName));
                LOGGER.error("*********** " + result + " *************");
            }


            return result;
        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
            return "";
        }
    }

    private static String getDatePart(Calendar calendar, String part) throws Exception {
        try {
            DateFormatParts formatPart = DateFormatParts.getOrNull(part);

            if (formatPart != null) {
                switch (formatPart) {
                    case yyyy: {
                        return new SimpleDateFormat("yyyy").format(calendar.getTime());
                    }
                    case yy: {
                        return new SimpleDateFormat("yy").format(calendar.getTime());
                    }
                    case monthed:
                    case month:
                    case monthec:
                    case mm: {
                        return new SimpleDateFormat("MM").format(calendar.getTime());
                    }
                    case m: {
                        return new SimpleDateFormat("M").format(calendar.getTime());
                    }
                    case dd: {
                        return new SimpleDateFormat("dd").format(calendar.getTime());
                    }
                    case d: {
                        return new SimpleDateFormat("d").format(calendar.getTime());
                    }
                    case hh: {
                        return new SimpleDateFormat("hh").format(calendar.getTime());
                    }
                    case h: {
                        int hour = calendar.get(Calendar.HOUR_OF_DAY);
                        if (hour == 0) {
                            return String.valueOf(12);
                        }
                        if (hour > 12) {
                            return String.valueOf(hour - 12);
                        } else {
                            return String.valueOf(hour);
                        }
                    }
                    case HH: {
                        return new SimpleDateFormat("HH").format(calendar.getTime());
                    }
                    case H: {
                        return new SimpleDateFormat("H").format(calendar.getTime());
                    }
                    case MM: {
                        return new SimpleDateFormat("mm").format(calendar.getTime());
                    }
                    case M: {
                        return new SimpleDateFormat("m").format(calendar.getTime());
                    }
                    case SS: {
                        return new SimpleDateFormat("ss").format(calendar.getTime());
                    }
                    case S: {
                        return new SimpleDateFormat("s").format(calendar.getTime());
                    }
                }
            }

            return "${" + part + "}";
        } catch (Exception e){
            LOGGER.error(e.getMessage(), e);
            return null;
        }
    }

    private static String unescapeJSONSpecials(String value){
        value = value.replaceAll("\"", "\\\\\"");
        value = value.replaceAll("\'", "\\\\\'");
        value = value.replaceAll("\b", "\\\\\b");
        value = value.replaceAll("\f", "\\\\\f");
        value = value.replaceAll("\n", "\\\\\n");
        value = value.replaceAll("\r", "\\\\\r");
        value = value.replaceAll("\t", "\\\\\t");
        return value;
    }

    @PreDestroy
    public void destroy() {
        executor.shutdown();
    }

}
